/*
 * HTTPRequestMessage.java
 *
 * Created on September 11, 2005, 1:38 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package org.msblabs.http;

import java.io.BufferedReader;
import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;

/**
 *
 * @author acidus
 */
public class HTTPRequestMessage extends HTTPMessage{
    
    
    private String method;
    private String resource;
    private String version;
    private String host;
    private String port;
    private String rawResource;
    
    private boolean valid;
    
    private static String [] httpMethods = {"OPTIONS",
                                            "GET",
                                            "HEAD",
                                            "POST",
                                            "PUT",
                                            "DELETE",
                                            "TRACE",
                                            "CONNECT"}; 
    
    public HTTPRequestMessage() {
        super();
        method = "";
        resource="";
        rawResource = "";
        version="";
        host = "";
        port = "";
    }
    
    //overwrite the setAction;
    public boolean setAction(String s) {
        action = s;
        valid = false;    
        String [] tmp = null;
        for(String method: httpMethods) {
            //if its a valid HTTP method...
            if(s.startsWith(method) || s.startsWith(method.toLowerCase())) {
                tmp = s.split("\\s");
                method = tmp[0];
		resource = tmp[1];
                //convert proxy form of resource to actual form (interface remove hostname)
                rawResource = new String(resource);
                int i = 0;
                if( (i = resource.indexOf("://")) > 0) {
                    i = resource.indexOf("/", i + 3);
                    resource = resource.substring(i);
                }
                if(tmp.length > 2) {
                    version = tmp[2];
                }
                StringBuilder sb = new StringBuilder();
                sb.append(method);
                sb.append(' ');
                sb.append(resource);
                sb.append(' ');
                sb.append(version);
                action = sb.toString();
                return true;
            }
        }
        return false;
    }

    //given an input from the client, and a content length
    //receive and set the body of the request (POST data, etc)
    
    public void extractBody(BufferedReader br) throws IOException {
        
        String n = getHeader("Content-Length");
        int len =0;
        if(!n.equals("")) {
            len = Integer.parseInt(n);
        }
        if(len > 0) {
            body = new byte[len];
            int count=0;
            byte b;
            while( count < len) {
                body[count] =  (byte) br.read();
                count++;
            }
        }
    }
   
    public void extractHeaders(BufferedReader br) throws IOException {
   
        String inputLine = "";
        while (!(inputLine = br.readLine()).equals("")) {
            this.addHeader(inputLine);
        }
    }

    public Socket makeConnection() throws UnknownHostException, IOException {
        //finalize host/port info from action and headers
        finalizeRemoteInfo();
        return new Socket(host, Integer.parseInt(port));
    }
   
    private void finalizeRemoteInfo() {
        for(HTTPHeader h : msgHeaders) {
            //is it the Host line?
            if(h.getName().equalsIgnoreCase("Host")) {
                String [] tmp = h.getValue().split(":");
                //we either have "www.site.com" or
                //we have        "www.site.com:8080"
                host = tmp[0].trim();
                if(tmp.length == 2) {
                    port = tmp[1].trim();
                } else {
                    port = "80";
                }
                break; //stop looking
            }
        }
    }
    
    public String getResource() {
        return resource;
    }

    public String getRawResource() {
        return rawResource;
    }

    public String getHost() {
        return host;
    }
    
}
