/*
 * LBProxyThread.java
 *
 * Created on September 12, 2005, 1:15 AM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package org.msblabs.http;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.UnsupportedEncodingException;
import java.net.Socket;
import java.net.URLDecoder;
import java.net.UnknownHostException;

/**
 *
 * @author acidus
 */
public class LBProxyThread  extends Thread {

    
    
    /** a reference to the calling proxy */
    private LBProxy theProxy; 
    
    
    /** connection between client and this proxy*/
    private Socket client; 
    /** connection between this proxy and remote host*/
    private Socket remote;
    
    //---------------------- used to report XSS attacks
    private String detectedValue;
    private String badParam;
    private String badUrl;

//-------------------------------------------------------- Constructor    
    
    public LBProxyThread (Socket sock, LBProxy p) { 
        client = sock; 
        theProxy = p;
    } 

    
//------------------------------------------------------- MISC Funcs

    
    private boolean internalImageURL(HTTPRequestMessage msg) {
        if(msg.getRawResource().startsWith("http://this-is-not-a-real-url-it-is-for-line-braker.com")) {
            return true;
        }
        return false;
    }
    
    
    private boolean safeURL(String s) throws UnsupportedEncodingException {
        //first things first, get to the parameters
        // http://...blah.../?PARAMS-PARAMS-PARAMS#...blah...
        
        int start = s.indexOf('?');
        int end = s.lastIndexOf('#');
        
        if(start == -1 || start+1 >= s.length())
            return true; //all urls without params are defined as safe
        
        if(end == -1) {
            //all the way to the last character
            end = s.length();
        }
        //And everything in between... 
        String params = (URLDecoder.decode(s.substring(start+1, end), "UTF-8")).toUpperCase();
        for(String xssString : LBProxy.xssSignatures) {
            if(params.indexOf(xssString.toUpperCase()) > 0) {
                detectedValue = LBProxy.sanitize(xssString);
                badUrl = s;
               
                return false;
            }
        }
        return true;
    }
    
//------------------------------------------------ RUN OF THE THREAD    
  
    public void run() { 
        HTTPRequestMessage request = null;
        try { 
            
            BufferedReader fromClient = new BufferedReader(
                                new InputStreamReader(client.getInputStream()));

            request = new HTTPRequestMessage();
            
            String inputLine = fromClient.readLine();
            if(!request.setAction(inputLine)) {

                PrintStream toClient = new PrintStream(
                              client.getOutputStream());
                System.out.println("Thread received invalid HTTP Request!");
                toClient.print("HTTP/1.1 200 OK\r\n");
                toClient.print("Content-type: text/html\r\n");
                toClient.print("\r\n");
                toClient.print("<h3>BADLY FORMED REQUEST</h3>");
                client.close();
                return;
            }
            System.out.println(inputLine);
            //check for XSS attack
            if(!safeURL(request.getRawResource())) {
                //have we already marked this link as ok?
                if(!theProxy.removePendingURL(badUrl)) {
                    //if not, then return a warning page
                
                    PrintStream toClient = new PrintStream(
                                  client.getOutputStream());
                    //add this "bad" link to the list of outstanding "bad" links
                    //so when the user clicks it, LB doesn't return a warning page
                    //again
                    theProxy.addPendingURL(badUrl);
                    toClient.print(LBProxy.generateWarningPage(badUrl, detectedValue));
                    client.close();
                    return;
                }
            }
            
            if(this.internalImageURL(request)) {
                String s = request.getResource();
                byte [] img = null;
                if(s.equals("/line-breaker-logo.png")) {
                    img = LBProxy.generateImage("/datafiles" + s);
                }
                // SPI logo stuff here
                if(s.equals("/spi-logo.png")) {
                    img = LBProxy.generateImage("/datafiles" + s);
                }
                if(img != null) {
                    MyByteArray replyMsg = new MyByteArray();
                    //create out header
                    StringBuilder sb = new StringBuilder();
                    sb.append("HTTP/1.1 200 OK");
                    sb.append("\r\n");
                    sb.append("Connection: close");
                    sb.append("\r\n");        
                    sb.append("Content-type: image/png");
                    sb.append("\r\n");
                    sb.append("Content-Length: ");
                    sb.append(img.length);
                    sb.append("\r\n");
                    sb.append("\r\n");
                    //add our header to reply
                    replyMsg.add(sb.toString());
                    //add our img payload
                    replyMsg.add(img);
                    
                    OutputStream toClient = client.getOutputStream();
                    //send the image
                    toClient.write(replyMsg.getBytes());
                    toClient.flush();
                    client.close();
                    return;
                }
                    
                LBProxy.generateImage(request.getResource());
            }
            
            //--- create the request, and format it as a proxy should
            
            //get the headers
            request.extractHeaders(fromClient);
            //get body data (from POST, PUT, etc) if any exists
            request.extractBody(fromClient);
            //get rid of headers that should not be passed down the chain
            request.removeUnpassableHeaders();
            //add the LineBreaker Header
            request.addHeader(LBProxy.lbHeader);
            //LineBreaker does not support persistent connections (yet)
            request.addHeader("Connection: close");
            
            //Create a connection to the server, and get teh Streams

            remote = request.makeConnection();
            
            InputStream  fromRemote  = remote.getInputStream();
            OutputStream toRemote = remote.getOutputStream();            


            //write the request to the server
            toRemote.write(request.getMessage());
            
            //================================================== HANDLE RESPONSE            

            //Read in the entire response from the server
            int count =0;
            byte [] buffer = new byte[4096];
            MyByteArray mba = new MyByteArray();
            while ((count = fromRemote.read(buffer)) != -1) {
                mba.add(buffer, count);
            }
            byte[] reply = mba.getBytes();

            //create the HTTP response message by parsing this byte array
            //and automatically extracting the headers and payload
            HTTPMessage response = new HTTPMessage(reply);

            //close the connection to the remote server
            fromRemote.close();
            remote.close();
            remote = null;
            
            //get rid of headers that should not be passed down the chain
            response.removeUnpassableHeaders();
            //add the LineBreaker Header
            response.addHeader(LBProxy.lbHeader);
            //LineBreaker does not support persistent connections (yet)
            response.addHeader("Connection: close");
            
                
            OutputStream toClient = client.getOutputStream();
            //write the response to the client
            toClient.write(response.getMessage());
            toClient.flush();
            client.close();


        } catch (UnknownHostException uhe) {
            System.out.print("Unknown host *" + request.getHost());
            if(request.getHost().equals("")) {
                System.out.print("\"" + request.getHost() + "\"");
            }
            System.out.println("");
        } 
        catch ( Exception e ) {
            
            System.out.println( "I/O error " + e ); 
            e.printStackTrace();
            System.exit(1);
       } finally  {
           try {
               if(remote != null)
                   remote.close();
               client.close();
           } catch (Exception e2 ) {
            //do nothing,. we are dying}
           }
       }        

        //System.out.println("Done processing");
    } 
    
}
